from django.urls import include
from django.conf.urls import url
from rest_framework import routers

from djangorestfilemanager.views import FileModelViewSet

router = routers.DefaultRouter()
router.register(r'files', FileModelViewSet)

urlpatterns = [
    url(r'^', include(router.urls)),
]



from django.contrib.auth import get_user_model
from django.core.files.base import ContentFile
from django.test import override_settings
from rest_framework import status
from rest_framework.test import APITestCase, APIClient
from djangorestfilemanager.models import File



@override_settings(ROOT_URLCONF='djangorestfilemanager.urls', PUB_SUB_EMIT=False)
class TestUploadFile(APITestCase):

    def setUp(self):
        self.client = APIClient()
        self.file = ContentFile(b'content', name='plain.txt')
        self.data = {
            'file': self.file,
            'name': 'test',
            'username': 'Luke SKW',
            'type': 'test'
        }
        User = get_user_model()
        self.user = User.objects.create(username='lskywalker', is_staff=True, is_superuser=True, is_active=True)

    def tearDown(self):
        for file in File.objects.all():
            file.file.delete()

    def test_upload_not_authenticated_file(self):
        file = {'file': (str(self.file.name), self.file, 'text/plain', {'Expires': '0'})}
        response = self.client.post('/files/', data=self.data, file=file)
        self.assertEqual(status.HTTP_403_FORBIDDEN, response.status_code)
        self.assertEqual(0, File.objects.all().count())

    def test_upload_file(self):
        self.client.force_authenticate(user=self.user)
        file = {'file': (str(self.file.name), self.file, 'text/plain', {'Expires': '0'})}
        response = self.client.post('/files/', data=self.data, file=file)
        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        self.assertIsNone(response.data.get('file', None))
        self.assertIsNotNone(response.data.get('uuid', None))
        self.assertEqual(1, File.objects.all().count())



from setuptools import setup, find_packages

setup(name='djangorestfilemanager',
      version='0.3.5',
      description='Django REST File Manager',
      url='https://gitlab.com/kas-factory/packages/django-rest-file-manager',
      author='Avelino @ KF',
      author_email='avelino@kasfactory.net',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      package_data={
          'djangorestfilemanager': ['djangorestfilemanager/*',
                                    'djangorestfilemanager/migrations/*',
                                    'djangorestfilemanager/locale/*',
                                    'djangorestfilemanager/locale/es/*',
                                    'djangorestfilemanager/locale/en/*',
                                    'djangorestfilemanager/locale/en/LC_MESSAGES/*',
                                    'djangorestfilemanager/locale/es/LC_MESSAGES/*',
                                    'djangorestfilemanager/locale/en/LC_MESSAGES/django.po',
                                    'djangorestfilemanager/locale/es/LC_MESSAGES/django.po',
                                    'djangorestfilemanager/locale/en/LC_MESSAGES/django.mo',
                                    'djangorestfilemanager/locale/es/LC_MESSAGES/django.mo',
                                    'djangorestfilemanager/test/*'
                                    ]
      },
      install_requires=[
            'boto3>=1.10.35',
            'Django>=2.0',
            'djangorestframework>=3.0',
            'django-filter>=2.2.0',
            'django-storages>=1.8',
            'PyJWT>=1.7.1',
            'pytest-django>=3.7.0'
      ],
      zip_safe=False
      )



# -*- coding: utf-8 -*-
import datetime
import time
import uuid
from django.db import models
from django.utils.translation import gettext as _
from django.template.defaultfilters import slugify
from django.conf import settings
import os
from storages.backends.s3boto3 import S3Boto3Storage
from django.core.files.storage import default_storage

from djangorestfilemanager.settings import UPLOADS_DIR


class RestFileManagerStorage(S3Boto3Storage):

    def get_available_name(self, name, max_length=None):
        now = time.time()
        stamp = datetime.datetime.fromtimestamp(now).strftime('%Y-%m-%d-%H-%M-%S-%f')
        return slugify('{}_{}'.format(name, stamp))


def get_file_system_storage():
    if getattr(settings, 'S3_STORAGE'):
        return RestFileManagerStorage(location=getattr(settings, 'AWS_LOCATION', 'files'))
    else:
        return default_storage


def set_storage_file_dir(instance, filename):
    return os.path.join(UPLOADS_DIR, instance.type, filename)


class File(models.Model):
    uuid = models.UUIDField(
        default=uuid.uuid4,
        primary_key=True
    )
    file = models.FileField(
        verbose_name=_('File'),
        storage=get_file_system_storage(),
        upload_to=set_storage_file_dir
    )
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=100
    )
    username = models.CharField(
        _('Username'),
        max_length=100
    )
    creation_date = models.DateTimeField(
        verbose_name=_('Creation date'),
        auto_now_add=True,
    )
    last_mod_date = models.DateTimeField(
        verbose_name=_('Creation date'),
        auto_now=True,
    )
    origin = models.CharField(
        verbose_name=_('Origin'),
        blank=True,
        null=True,
        max_length=3,
        default='API'
    )
    type = models.CharField(
        verbose_name=_('Type'),
        blank=True,
        null=True,
        max_length=10,
        default='file'
    )
    permission = models.CharField(
        verbose_name=_('Permission'),
        max_length=200,
        blank=True,
        null=True,
        default=''
    )
    share = models.BooleanField(
        verbose_name=_('Share'),
        default=False
    )

    class Meta:
        verbose_name = _('File')
        verbose_name_plural = _('Files')

    def __str__(self):
        return '{}-{}'.format(self.name, self.type)

    def to_dict(self):
        return {
            'uuid': str(self.uuid),
            'name': self.name,
            'origin': self.origin,
            'type': self.type,
            'username': self.username
        }
